//
// Copyright (c) 2005-2011 StreamBase Systems, Inc. All rights reserved.
//
package com.streambase.sb.contrib.adapter.sbmonitor;

import java.util.ArrayList;

import com.streambase.sb.CompleteDataType;
import com.streambase.sb.Schema;
import com.streambase.sb.Timestamp;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.streambase.sb.monitor.ModuleInfo;

public class ModuleInfoSchemaHelper {
	// names for ModuleInfo
	public static final String TIME="time";
	public static final String NAME="moduleName";
	public static final String QUEUES="queues";
	
	// names for QueueInfo
	public static final String QUEUE_NAME="queueName";
	public static final String QUEUE_BATCHLATENCY="batchLatency";
	public static final String QUEUE_BATCHPROCESSINGTIME="batchProcessingTime";
	public static final String QUEUE_CURRENTQUEUELENGTH="currentQueueLength";
	public static final String QUEUE_MAXQUEUELENGTH="maxQueueLength";
	
	public static final Schema.Field[] QUEUE_FIELDS = {
		new Schema.Field(QUEUE_NAME, CompleteDataType.forString()),
		new Schema.Field(QUEUE_BATCHLATENCY, CompleteDataType.forInt()),
		new Schema.Field(QUEUE_BATCHPROCESSINGTIME, CompleteDataType.forInt()),
		new Schema.Field(QUEUE_CURRENTQUEUELENGTH, CompleteDataType.forInt()),
		new Schema.Field(QUEUE_MAXQUEUELENGTH, CompleteDataType.forInt())
	};
	
	public static Schema QUEUE_SCHEMA=new Schema(null, QUEUE_FIELDS);
	
	public static final int IDX_FIELD_QUEUE_NAME = QUEUE_SCHEMA.getFieldIndex(QUEUE_NAME);
	public static final int IDX_FIELD_QUEUE_BATCHLATENCY = QUEUE_SCHEMA.getFieldIndex(QUEUE_BATCHLATENCY);
	public static final int IDX_FIELD_QUEUE_BATCHPROCESSINGTIME = QUEUE_SCHEMA.getFieldIndex(QUEUE_BATCHPROCESSINGTIME);
	public static final int IDX_FIELD_QUEUE_CURRENTQUEUELENGTH = QUEUE_SCHEMA.getFieldIndex(QUEUE_CURRENTQUEUELENGTH);
	public static final int IDX_FIELD_QUEUE_MAXQUEUELENGTH = QUEUE_SCHEMA.getFieldIndex(QUEUE_MAXQUEUELENGTH);
	
	public static final Schema.Field[] FIELDS = {
		new Schema.Field(TIME, CompleteDataType.forTimestamp()),
		new Schema.Field(NAME, CompleteDataType.forString()),
		new Schema.Field(QUEUES, CompleteDataType.forList(CompleteDataType.forTuple(QUEUE_SCHEMA)))
	};
	
	public static Schema SCHEMA=new Schema(null, FIELDS);
	
	public static final int IDX_FIELD_TIME = SCHEMA.getFieldIndex(TIME);
	public static final int IDX_FIELD_NAME = SCHEMA.getFieldIndex(NAME);
	public static final int IDX_FIELD_QUEUES = SCHEMA.getFieldIndex(QUEUES);	
	
	public static Tuple createTuple() {
		return SCHEMA.createTuple();
	}
	
	public static Tuple createQueueTuple(ModuleInfo.QueueInfo q) throws TupleException {
		Tuple t=QUEUE_SCHEMA.createTuple();
		t.clear();
		
		t.setString(IDX_FIELD_QUEUE_NAME, q.getName());
		t.setInt(IDX_FIELD_QUEUE_BATCHLATENCY, q.getBatchLatency());
		t.setInt(IDX_FIELD_QUEUE_BATCHPROCESSINGTIME, q.getBatchProcessingTime());
		t.setInt(IDX_FIELD_QUEUE_CURRENTQUEUELENGTH, q.getCurrentQueueLength());
		t.setInt(IDX_FIELD_QUEUE_MAXQUEUELENGTH, q.getMaxQueueLength());
		
		return t;
	}
	
	public static Tuple createModuleTuple(Timestamp ts, ModuleInfo mod) throws TupleException {
		Tuple tuple = createTuple();
		tuple.clear();
		
		tuple.setTimestamp(IDX_FIELD_TIME, ts);
		tuple.setString(IDX_FIELD_NAME, mod.getName());
		ArrayList<Tuple> queues=new ArrayList<Tuple>();
		
		for (ModuleInfo.QueueInfo q : mod.queues()) {
			queues.add(createQueueTuple(q));
		}
		tuple.setList(IDX_FIELD_QUEUES, queues);
		
		return tuple;

	}
}
