//
// Copyright (c) 2005-2011 StreamBase Systems, Inc. All rights reserved.
//
package com.streambase.sb.contrib.adapter.sbmonitor;

import java.beans.*;

import com.streambase.sb.operator.parameter.*;

/**
 * A BeanInfo class controls what properties are exposed, add 
 * metadata about properties (such as which properties are optional), and access 
 * special types of properties that can't be automatically derived via reflection. 
 * If a BeanInfo class is present, only the properties explicitly declared in
 * this class will be exposed by StreamBase.
 */
public class SBMonitorAdapterBeanInfo extends SimpleBeanInfo {

	public static final String URI_PROPTERY_NAME="URI";	
	/*
	 * The order of properties below determines the order they are displayed within
	 * the StreamBase Studio property view. 
	 */
	public PropertyDescriptor[] getPropertyDescriptors() {
		try {
			PropertyDescriptor[] p = {
				new SBPropertyDescriptor("RemoteConnect", SBMonitorAdapter.class).displayName("Monitor Remote Server ").
					description("").optional(),
				new SBPropertyDescriptor(URI_PROPTERY_NAME, SBMonitorAdapter.class).displayName("StreamBase URI").
					description("The StreamBase URI for the server to monitor. Blank means the current server.").optional(),
	            new SBPropertyDescriptor("ReconnectInterval", SBMonitorAdapter.class).displayName("Reconnect Wait Interval")
                    .description("The number of seconds to wait before attempting to reconnect to the server").optional(),
				new SBPropertyDescriptor("EnableOperator", SBMonitorAdapter.class).displayName("Enable Operator Info").
					description("").optional(),
				new SBPropertyDescriptor("EnableModule", SBMonitorAdapter.class).displayName("Enable Module Info").
					description("").optional(),
				new SBPropertyDescriptor("EnableThread", SBMonitorAdapter.class).displayName("Enable Thread Info").
					description("").optional(),
				new SBPropertyDescriptor("EnableStream", SBMonitorAdapter.class).displayName("Enable Stream Info").
					description("").optional(),
				new SBPropertyDescriptor("EnableSystem", SBMonitorAdapter.class).displayName("Enable System Info").
					description("").optional()
			};

			return p;
		} catch (IntrospectionException e) {
			System.err.println("Failed to create property descriptors");
			e.printStackTrace();
			return null;
		}
	}

}
