//
// Copyright (c) 2010-2011 StreamBase Systems, Inc. All rights reserved.
//
package com.streambase.sb.contrib.adapter.sbmonitor;

import com.streambase.sb.CompleteDataType;
import com.streambase.sb.Schema;
import com.streambase.sb.Timestamp;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;


public class StatusSchemaHelper {
    public enum StatusType {
        CONNECTION,
        ERROR;
    }
    
    public enum StatusAction {
        CONNECTED,
        CONNECTING,
        DISCONNECTED,
        INITIALIZING,
        RESUMING,
        UNAVAILABLE,
        WARNING,
        FAILED,
        INFORMATION,
    }
    
    public static final Schema.Field[] FIELDS = {
        new Schema.Field("Type", CompleteDataType.forString()),
        new Schema.Field("Object", CompleteDataType.forString()),
        new Schema.Field("Action", CompleteDataType.forString()),
        new Schema.Field("Message", CompleteDataType.forString()),
        new Schema.Field("Time", CompleteDataType.forTimestamp()),
    };
    
    public static Schema SCHEMA = new Schema("StatusSchema", FIELDS);
    
    public static final int IDX_FIELD_TYPE = SCHEMA.getFieldIndex("Type");
    public static final int IDX_FIELD_OBJECT = SCHEMA.getFieldIndex("Object");
    public static final int IDX_FIELD_ACTION = SCHEMA.getFieldIndex("Action");
    public static final int IDX_FIELD_MESSAGE = SCHEMA.getFieldIndex("Message");
    public static final int IDX_FIELD_TIME = SCHEMA.getFieldIndex("Time");
    
    public static Tuple populateTuple(Tuple tuple, StatusType type, String object, StatusAction action, String message) throws TupleException {
        tuple.clear();    
        tuple.setString(IDX_FIELD_TYPE, type.toString());
        tuple.setString(IDX_FIELD_OBJECT, object);
        tuple.setString(IDX_FIELD_ACTION, action.toString());
        tuple.setString(IDX_FIELD_MESSAGE, message);
        // Set the time for this tuple
        tuple.setTimestamp(IDX_FIELD_TIME, Timestamp.now());
        return tuple;
    }
}
