//
// Copyright (c) 2005-2011 StreamBase Systems, Inc. All rights reserved.
//
package com.streambase.sb.contrib.adapter.sbmonitor;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;

import com.streambase.sb.CompleteDataType;
import com.streambase.sb.Schema;
import com.streambase.sb.Timestamp;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.streambase.sb.monitor.SystemInfo;
import com.streambase.sb.util.Msg;

public class SystemInfoSchemaHelper {
	
	// GC information is introduced in 7.1.
	// To support older versions use reflection to see if the GC info is available
	private static boolean hasGCInfoMethod=true;
	// method to get the array of GCInfo objects
	private static Method GCInfoMethod=null;
	
	// methods to get data out of the GCInfo object
	private static Method GCInfoGetNameMethod=null;
	private static Method GCInfoGetCountMethod=null;
	private static Method GCInfoGetTimeMethod=null;

	// Get and cache all the methods 
	private static Method getGCInfoMethod() {
		if (GCInfoMethod != null)
			return GCInfoMethod;
		
		if (!hasGCInfoMethod)
			return null;
		
		try {
			// The method that returns the collection of GCInfo objects
			GCInfoMethod = SystemInfo.class.getMethod("getGCollectors");
			
			// Now get the methods to extract the data from the GCInfo object 
			Class<?>[] cls = SystemInfo.class.getClasses();
			for (Class<?> Clazz : cls) {
				if ("com.streambase.sb.monitor.SystemInfo$GCInfo".equals(Clazz.getName())) {
					
					GCInfoGetNameMethod = Clazz.getMethod("getName");
					GCInfoGetCountMethod = Clazz.getMethod("getGCCount");
					GCInfoGetTimeMethod = Clazz.getMethod("getGCTimeMS");
					break;
				}
			}
		} catch (Exception e) {
			hasGCInfoMethod=false;
			Msg.debug("Using SystemInfo class prior to getGCollectors");
			return null;
		}
		return GCInfoMethod;
	}
	
	public static final String TIME="time";
	public static final String CLIENTCOUNT="ClientCount";
	public static final String DEQUEUECLIENTCOUNT="dequeueClientCount";
	public static final String FREEMEMORY="freeMemory";
	public static final String MAXMEMORY="maxMemory";
	public static final String TIMERUNNING="timeRunning";
	public static final String TOTALMEMORY="totalMemory";
	public static final String TOTALNUMBERTHREADS="totalNumberThreads";
	public static final String USEDMEMORY="usedMemory";
	
	public static final String GCCOLLECTORS="gcCollectors";
	public static final String GCNAME="name";
	public static final String GCCOUNT="count";
	public static final String GCTIMEMS="timeMS";
	
	public static final Schema.Field[] GC_FIELDS = {
		new Schema.Field(GCNAME, CompleteDataType.forString()),
		new Schema.Field(GCCOUNT, CompleteDataType.forLong()),
		new Schema.Field(GCTIMEMS, CompleteDataType.forLong()),
	};
	public static Schema GC_SCHEMA=new Schema(null, GC_FIELDS);
	
	public static final int IDX_GCNAME = GC_SCHEMA.getFieldIndex(GCNAME);
	public static final int IDX_GCCOUNT = GC_SCHEMA.getFieldIndex(GCCOUNT);
	public static final int IDX_GCTIMEMS = GC_SCHEMA.getFieldIndex(GCTIMEMS);
	
	public static final Schema.Field[] FIELDS = {
		new Schema.Field(TIME, CompleteDataType.forTimestamp()),
		new Schema.Field(TIMERUNNING, CompleteDataType.forInt()),
		new Schema.Field(CLIENTCOUNT, CompleteDataType.forInt()),
		new Schema.Field(DEQUEUECLIENTCOUNT, CompleteDataType.forInt()),
		new Schema.Field(TOTALNUMBERTHREADS, CompleteDataType.forInt()),
		new Schema.Field(TOTALMEMORY, CompleteDataType.forLong()),
		new Schema.Field(FREEMEMORY, CompleteDataType.forLong()),
		new Schema.Field(MAXMEMORY, CompleteDataType.forLong()),
		new Schema.Field(USEDMEMORY, CompleteDataType.forLong()),
		new Schema.Field(GCCOLLECTORS, CompleteDataType.forList(CompleteDataType.forTuple(GC_SCHEMA)))
	};
	
	public static Schema SCHEMA=new Schema(null, FIELDS);
	
	public static final int IDX_FIELD_TIME = SCHEMA.getFieldIndex(TIME);
	public static final int IDX_FIELD_CLIENTCOUNT = SCHEMA.getFieldIndex(CLIENTCOUNT);
	public static final int IDX_FIELD_DEQUEUECLIENTCOUNT = SCHEMA.getFieldIndex(DEQUEUECLIENTCOUNT);
	public static final int IDX_FIELD_FREEMEMORY = SCHEMA.getFieldIndex(FREEMEMORY);
	public static final int IDX_FIELD_MAXMEMORY = SCHEMA.getFieldIndex(MAXMEMORY);
	public static final int IDX_FIELD_TIMERUNNING = SCHEMA.getFieldIndex(TIMERUNNING);
	public static final int IDX_FIELD_TOTALMEMORY = SCHEMA.getFieldIndex(TOTALMEMORY);
	public static final int IDX_FIELD_TOTALNUMBERTHREADS = SCHEMA.getFieldIndex(TOTALNUMBERTHREADS);
	public static final int IDX_FIELD_USEDMEMORY = SCHEMA.getFieldIndex(USEDMEMORY);
	public static final int IDX_GCCOLLECTORS = SCHEMA.getFieldIndex(GCCOLLECTORS);
	
	public static Tuple createTuple() {
		return SCHEMA.createTuple();
	}
	
	@SuppressWarnings("unchecked")
	public static Tuple createSystemTuple(Timestamp ts, SystemInfo sy) throws TupleException {
		Tuple tuple = createTuple();
		tuple.clear();
		
		tuple.setTimestamp(IDX_FIELD_TIME, ts);
		tuple.setInt(IDX_FIELD_CLIENTCOUNT, sy.getClientCount());
		tuple.setInt(IDX_FIELD_DEQUEUECLIENTCOUNT, sy.getDequeueClientCount());
		tuple.setLong(IDX_FIELD_FREEMEMORY, sy.getFreeMemory());
		tuple.setLong(IDX_FIELD_MAXMEMORY, sy.getMaxMemory());
		tuple.setInt(IDX_FIELD_TIMERUNNING, sy.getTimeRunning());
		tuple.setLong(IDX_FIELD_TOTALMEMORY, sy.getTotalMemory());
		tuple.setInt(IDX_FIELD_TOTALNUMBERTHREADS, sy.getTotalNumberThreads());
		tuple.setLong(IDX_FIELD_USEDMEMORY, sy.getUsedMemory());
		
		Method cm = getGCInfoMethod();
		if (cm != null) {
			try {
				// Get the GCInfo objects
				
				Collection<Object> gcs = (Collection<Object>)cm.invoke(sy);
				
				// Populate the tuple list with collector info
				ArrayList<Tuple> gcList = new ArrayList<Tuple>();
				for (Object myGCInfo : gcs) {
					Tuple subT = GC_SCHEMA.createTuple();
					String name= (String)GCInfoGetNameMethod.invoke(myGCInfo);
					subT.setString(IDX_GCNAME, name);
					long count = (Long)GCInfoGetCountMethod.invoke(myGCInfo);
					subT.setLong(IDX_GCCOUNT, count);
					long timeMS = (Long)GCInfoGetTimeMethod.invoke(myGCInfo);
					subT.setLong(IDX_GCTIMEMS, timeMS);
					gcList.add(subT);
				}
				tuple.setList(IDX_GCCOLLECTORS, gcList);
			} catch (Exception e) {
				Msg.debug(e);
				return tuple;
			}
		}
		return tuple;

	}
}
